use clap::Parser;
use config_diag::ConfigDiag;
use expect_exit::ExpectedWithError;

mod defs;
mod output;
mod parse;

use defs::Config;

#[derive(Debug, Parser)]
#[clap(author, version, about)]
struct Cli {
    /// The name of the JSON test data file.
    #[clap(short = 'f', long)]
    data_file: String,

    /// The name of the file to generate.
    #[clap(short, long)]
    output: String,

    /// Verbose operation; display diagnostic output.
    #[clap(short, long)]
    verbose: bool,

    /// The path to the template to use.
    template_path: String,
}

fn parse_args() -> Config {
    let args = Cli::parse();
    Config {
        data_path: args.data_file,
        output_path: args.output,
        template_path: args.template_path,
        verbose: args.verbose,
    }
}

fn main() {
    let cfg = parse_args();
    let template = parse::parse_template(&cfg)
        .or_exit_e(|| format!("Error parsing the {} template", cfg.template_path));
    let test_data = parse::parse_test_data(&cfg)
        .or_exit_e(|| format!("Error parsing the {} data file", cfg.data_path));
    output::write_output_file(&cfg, &template, &test_data)
        .or_exit_e_("Error generating the output file");
    cfg.diag_("Everything seems fine!");
}
